@@ -113,7 +113,7 @@ const char *ScriptChecks::chkCall(Line *line, ScriptImage *img)
 		p = strchr(proto, ':');
 		if(!p)
 			p = proto + strlen(proto);
-		diff = p - proto;
+		diff = (unsigned)(p - proto);
 		snprintf(proto + diff, sizeof(proto) - diff, "::%s", cp);
 		list = (Line *)img->getPointer(proto);
 	}
@@ -132,7 +132,7 @@ const char *ScriptChecks::chkCall(Line *line, ScriptImage *img)
 	while(NULL != (cp = getOption(list, &idx)))
 	{
 		snprintf(proto + len, sizeof(proto) - len, "=%s", cp);
-		len = strlen(proto);
+		len = (unsigned)strlen(proto);
 	}
 
 	if(len)
