@@ -115,9 +115,61 @@ const char *ScriptCompiler::getDefined(const char *token)
 
 void ScriptCompiler::commit(void)
 {
+	Name *scr, *target;
+	const char *last = "";
+	NamedEvent *ev, *from;
+	unsigned ecount;
+	const char *en;
+	char pbuf[65];
+	char *ep;
+
 	while(inccount)
 		include(incfiles[--inccount]);
 
+	while(mlist)
+	{
+		scr = getScript(mlist->source);
+		if(!scr)
+		{
+			if(mlist->source != last)
+				slog.error("include from %s not found", mlist->source);
+			goto cont;
+		}
+		ecount = 0;
+		en = mlist->prefix;
+		if(!*en)
+			en = "*:";
+
+		target = mlist->target;
+		from = scr->events;
+		while(from)
+		{
+			if(!strnicmp(from->name, mlist->prefix, strlen(mlist->prefix)))
+			{
+				ev = (NamedEvent *)alloc(sizeof(NamedEvent));
+				ev->line = from->line;
+				ev->name = from->name;
+				ev->type = from->type;
+				ev->next = target->events;
+				target->events = ev;
+				++ecount;
+			}
+
+			from = from->next;
+		}		
+		if(ecount)
+		{
+			setString(pbuf, sizeof(pbuf), en);
+			ep = strchr(pbuf, ':');
+			if(ep)
+				*ep = 0;
+			slog.debug("included %s from %s; %d events", pbuf, mlist->source, ecount);
+		}
+cont:		
+		last = mlist->source;
+		mlist = mlist->next;
+	}
+
 	ScriptImage::commit();
 }
 
@@ -479,6 +531,7 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 	char temp[64];
 	char *pmode = NULL;
 	char *ftoken = NULL;
+	char *filter;
 	bool bm = false;
 	bool first = true;
 	unsigned long addPmask = 0, subPmask = (unsigned long)~0;
@@ -487,6 +540,9 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 	const char *embed = NULL;
 	bool mscmd = false, execflag = false;
 	bool apps = false;
+	bool ripple = cmds->ripple;
+	bool wrapper = false;
+	merge_t *merge;
 
 	if(strstr(".bat.cmd", cp))
 		mscmd = true;
@@ -495,18 +551,28 @@ int ScriptCompiler::compile(istream *str, char *name, const char *scrname)
 		apps = true;
 	else if(strstr(exec_extensions, cp))
 	{
-		snprintf(namebuf, sizeof(namebuf), "%s::%s", exec_prefix, name);
+		snprintf(namebuf, sizeof(namebuf), "%s:%s", exec_prefix, name);
 		name = namebuf;
 		embed = exec_token;
+		wrapper = true;
 	}
 
+	if(!strnicmp(name, exec_prefix, strlen(exec_prefix)))
+		wrapper = true;
+
 	buffer = (char *)malloc(512);
 	bufsize = 512;
 
 	scrStream = str;
 
 	if(cp && !stricmp(cp, ".mac"))
+	{
 		bm = true;
+		ripple = false;
+	}
+
+	if(cp && !stricmp(cp, ".scr"))
+		ripple = false;
 
 	if(bm)
 		access = scrPROTECTED;
@@ -621,15 +687,17 @@ compile:
 			scrStream->getline(buffer + offset, bufsize - 1 - offset);
 			if(scrStream->eof())
 			{
-				if((embed) && !execflag)
+				if(wrapper && !execflag)
 				{
+					wrapper = false;
 					execflag = true;
 					embed = NULL;
 					apps = false;
 					setString(buffer, 16, "exec");
 				}
-				else if(embed || (!total && !count && !offset))
+				else if(wrapper || (!total && !count && !offset))
 				{
+					wrapper = false;
 					embed = NULL;
 					setString(buffer, 16, exit_token);
 				}
@@ -718,7 +786,7 @@ first:
 				continue;
 			}
 
-			if(!cmds->ripple && !embed && !apps)
+			if(!ripple && !embed && !apps)
   			  if(!stricmp(token, "private") || !stricmp(token,
 "protected") || !stricmp(token, "public") || !stricmp(token, "program")
 || !stricmp(token, "state") || !stricmp(token, "module"))
@@ -754,27 +822,27 @@ repname1:
 			}
 
 
-			if(!stricmp(token, "service") && !cmds->ripple && !embed && !apps)
+			if(!stricmp(token, "service") && !ripple && !embed && !apps)
 			{
 				ftoken = "options";
 				pmode = "public";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "func", 4) && !cmds->ripple && !embed && !apps && !strchr(token, ':'))
+			if(!strnicmp(token, "func", 4) && !ripple && !embed && !apps && !strchr(token, ':'))
 			{
 				ftoken = "fconst";
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!strnicmp(token, "proc", 4) && !cmds->ripple && !embed && !apps && !strchr(token, ':'))
+			if(!strnicmp(token, "proc", 4) && !ripple && !embed && !apps && !strchr(token, ':'))
 			{
 				pmode = "function";
 				goto repname1;
 			}
 
-			if(!stricmp(token, "catch") && !cmds->ripple && !embed && !apps)
+			if(!stricmp(token, "catch") && !ripple && !embed && !apps)
 			{
 				ftoken = "fconst";
 				name = getToken();
@@ -793,8 +861,13 @@ repname1:
 				break;
 			}
 
-			if((cmds->ripple || apps) && *token == '[')
+			if((ripple || apps) && *token == '[')
 			{
+				if(use_funcs)
+					pmode = "function";
+				else
+					pmode = "public";
+
 				name = token + 1;
 				cp = strchr(name, ']');
 				if(cp)
@@ -804,7 +877,7 @@ repname1:
 			}
 
 			tlen = (int)strlen(token);
-			if(token[tlen - 1] == ':' && !cmds->ripple && !apps && !embed)
+			if(token[tlen - 1] == ':' && !ripple && !apps && !embed)
 			{
 				token[tlen - 1] = 0;
 				name = token;
@@ -897,7 +970,7 @@ noexec:
 			snprintf(catchname, sizeof(catchname), "use.%s", token);
 			token = catchname;
 		}
-		else if(!stricmp(token, "virtual") && !embed && !cmds->ripple && scrStream == (istream *)&scrSource)
+		else if(!stricmp(token, "virtual") && !embed && (!ripple || use_macros) && scrStream == (istream *)&scrSource)
 		{
 			token = getToken();
 			if(!token)
@@ -911,12 +984,62 @@ noexec:
 
 			snprintf(temp, sizeof(temp), "virtual.%s", token);
 			if(!cmds->getLast(temp))
-				cmds->setValue(temp, "none");
+			{
+				cp = getToken();
+				if(!cp || !*cp)
+					cp = "none";
+				cmds->setValue(temp, "cp");
+			}
 
 			incfiles[inccount++] = alloc(token);
 			continue;
 		}
-		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource && !cmds->ripple && !apps && !embed)
+		else if(!stricmp(token, "include") && use_merge && !apps && !embed)
+		{
+			token = getToken();
+			if(!token)
+				continue;
+
+			if(!strchr(token, ':'))
+			{
+				snprintf(temp, sizeof(temp), name);
+				cp = strchr(temp, ':');
+				if(cp)
+					*cp = 0;
+				addString(temp, sizeof(temp), "::");
+				addString(temp, sizeof(temp), token);
+				token = temp;
+			}
+			token = alloc(token);
+			
+			filter = getToken();
+			for(;;)
+			{
+				if(filter)
+				{
+					snprintf(temp, sizeof(temp), "%s", filter);
+					cp = strchr(temp, ':');
+					if(cp)
+						*cp = 0;
+					addString(temp, sizeof(temp), ":");
+					filter = alloc(temp);
+				}
+				else
+					filter = "";
+
+				merge = (merge_t *)alloc(sizeof(merge_t));
+				merge->next = mlist;
+				merge->target = script;
+				merge->source = token;
+				merge->prefix = filter;
+				mlist = merge;
+				filter = getToken();
+				if(!filter)
+					break;
+			}
+			continue;
+		}
+		else if((!stricmp(token, "requires") || !stricmp(token, "import")) && scrStream == (istream *)&scrSource && (!ripple || use_macros) && !apps && !embed)
 		{
 			token = getToken();
 			if(!token)
@@ -937,27 +1060,27 @@ noexec:
 		if(!token)
 			continue;
 
-		if(*token == '%' && !cmds->ripple)
+		if(*token == '%' && !ripple)
 		{
 			pretoken = token;
 			token = "expr";
 		}
-		else if(*token == '&' && !cmds->ripple && !apps)
+		else if(*token == '&' && !ripple && !apps)
 		{
 			pretoken = token;
 			token = "call";
 		}
-		else if(!strnicmp(token, "*::", 3) && !cmds->ripple && !apps)
+		else if(!strnicmp(token, "*::", 3) && !ripple && !apps)
 		{
 			pretoken = token + 3;
 			token = "call";
 		}
-		else if(!strnicmp(token, "::", 2) && !cmds->ripple && !apps)
+		else if(!strnicmp(token, "::", 2) && !ripple && !apps)
 		{
 			pretoken = token + 2;
 			token = "call";
 		}
-		else if(strstr(token, "::") && !cmds->ripple && !apps)
+		else if(strstr(token, "::") && (!ripple || use_macros) && !apps)
 		{
 			pretoken = token;
 			token = "call";
@@ -1101,27 +1224,24 @@ noexec:
 
 		addmask = submask = 0;
 
-		if(!cmds->ripple && !apps)
+		if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !stricmp(command, "foreach"))
 		{
-			if(!stricmp(command, "repeat") || !stricmp(command, "for") || !stricmp(command, "do") || !stricmp(command, "foreach"))
-			{
-				if(!looplevel)
-					++loopid;
-				++looplevel;
-				line->loop = loopid * 128 + looplevel;
-			}
+			if(!looplevel)
+				++loopid;
+			++looplevel;
+			line->loop = loopid * 128 + looplevel;
+		}
 
-			if(!stricmp(command, "loop") || !strnicmp(command, "loop.", 5))
+		if(!stricmp(command, "loop") || !strnicmp(command, "loop.", 5))
+		{
+			line->loop = loopid * 128 + looplevel;
+			if(!looplevel)
 			{
-				line->loop = loopid * 128 + looplevel;
-				if(!looplevel)
-				{
-					slog.error("%s(%d): loop nesting error", filename, line->lnum);
-					continue;
-				}
-				else
-					--looplevel;
+				slog.error("%s(%d): loop nesting error", filename, line->lnum);
+				continue;
 			}
+			else
+				--looplevel;
 		}
 
 		memcpy(line->args, &args, sizeof(char *) * argc);
@@ -1192,11 +1312,13 @@ noexec:
 		execflag = false;
 		if(apps && apps_prefix)
 			snprintf(namebuf, sizeof(namebuf), "%s::%s", apps_prefix, name);
-		else if(strstr(name, "::") || cmds->ripple || apps)
+		else if(strstr(name, "::") || (ripple && !use_prefix) || apps)
 			snprintf(namebuf, sizeof(namebuf), "%s", name);
 		else
 			snprintf(namebuf, sizeof(namebuf), "%s::%s", basename, name);
 		name = namebuf;
+		if(!strnicmp(name, "exec:", 5) && exec_funcs)
+			pmode = "function";
 		goto compile;
 	}
 	scrSource.close();
@@ -1220,9 +1342,9 @@ noexec:
 			line = line->next;
 		}	
 		if(initkey == SCRIPT_INDEX_SIZE && initcount)
-			slog.info("compiled %s; %d steps", init->name, initcount);
+			slog.info("compiled %s initializer; %d steps", init->name, initcount);
 		else if(initcount)
-			slog.info("compiled %s; %d steps", init->name, initcount);
+			slog.info("compiled %s initializer; %d steps", init->name, initcount);
 		init->next = index[initkey];
 		index[initkey] = init;
 	}
