@@ -148,6 +148,8 @@ ScriptCommand()
 			(Check)&ScriptChecks::chkIgnore},
 		{"options", false, (Method)&ScriptMethods::scrNop,
 			(Check)&ScriptChecks::chkIgnore},
+		{"keywords", false, (Method)&ScriptMethods::scrNop,
+			(Check)&ScriptChecks::chkKeywords},
 		{"sequence", true, (Method)&ScriptMethods::scrSequence,
 			(Check)&ScriptChecks::chkSequence},
 		{"const", true, (Method)&ScriptMethods::scrConst,
