@@ -72,6 +72,10 @@ size_t Script::pagesize = 1024;
 unsigned Script::symsize = 64;
 unsigned Script::symlimit = 960;
 char Script::decimal = '.';
+bool Script::use_macros = false;
+bool Script::use_prefix = false;
+bool Script::use_merge = false;
+bool Script::use_funcs = false;
 const char *Script::plugins = NULL;
 const char *Script::altplugins = NULL;
 
@@ -87,10 +91,12 @@ const char *Script::apps_extensions = ".app.exec.apps";
 const char *Script::exec_extensions = ".sh";
 #endif
 const char *Script::exec_token = "script:";
-const char *Script::exec_prefix = "exec";
+const char *Script::exec_prefix = "exec:";
 const char *Script::exit_token = "exit";
 const char *Script::apps_prefix = "libexec";
 
+bool Script::exec_funcs = false;
+
 Script::Test *Script::test = NULL;
 Script::Fun *Script::ifun = NULL;
 Script::Package *Script::Package::first = NULL;
@@ -158,6 +164,21 @@ ScriptSysRegistry::ScriptSysRegistry()
 #endif
 }
 
+bool Script::isScript(Name *scr)
+{
+	const char *ext = strrchr(scr->filename, '.');
+	if(!ext)
+		return false;
+
+	if(!stricmp(ext, ".scr"))
+		return true;
+
+	if(!stricmp(ext, ".mac"))
+		return true;
+
+	return false;
+}
+
 bool Script::isSymbol(const char *id)
 {
 	if(*id == '%' || *id == '&' || *id == '@')
