@@ -58,6 +58,7 @@
 #define	SCRIPT_ROUTE_SLOTS 16
 #define	SCRIPT_EXEC_WRAPPER
 #define	SCRIPT_APPS_WRAPPER
+#define	SCRIPT_RIPPLE_LEVEL 2
 
 #ifndef CCXX_PACKING
 #if defined(__GNUC__)
@@ -249,6 +250,7 @@ public:
 	static unsigned symsize;
 	static unsigned symlimit;
 
+	static bool isScript(Name *scr);
 	static bool isSymbol(const char *id);
 	static bool use(const char *name);
 	static unsigned getIndex(const char *id);
@@ -261,11 +263,16 @@ public:
 	static unsigned storage(Symbol *sym);
 	static void clear(Symbol *sym);
 	static char decimal;
+	static bool use_macros;
+	static bool use_prefix;
+	static bool use_merge;
+	static bool use_funcs;
 	static const char *plugins;
 	static const char *altplugins;
 	static const char *access_user;
 	static const char *access_pass;
 	static const char *access_host;
+	static bool exec_funcs;
 	static const char *exec_extensions;
 	static const char *exec_token;
 	static const char *exec_prefix;
@@ -475,7 +482,7 @@ protected:
 	 * @return true if keyword syntax used.
 	 * @param line record to examine in check routine.
 	 */
-	bool hasKeywords(Line *line);
+	static bool hasKeywords(Line *line);
 
 public:
 	/**
@@ -530,7 +537,7 @@ protected:
 	 * @param line record to examine in check routine.
 	 * @param list of .members...
 	 */
-	bool useMember(Line *line, const char *list);
+	static bool useMember(Line *line, const char *list);
 
 	/**
 	 * Get an option to examine.
@@ -539,7 +546,7 @@ protected:
 	 * @param line record to examine.
 	 * @param pointer to index.  Start at 0.
 	 */
-	const char *getOption(Line *line, unsigned *index);
+	static const char *getOption(Line *line, unsigned *index);
 
 	/**
 	 * Load a set of keywords into the system keyword table.  This
@@ -618,6 +625,7 @@ protected:
 	
 	virtual void attach(ScriptInterp *interp);
 	virtual void detach(ScriptInterp *interp);
+	virtual bool reload(ScriptCompiler *img);
 	virtual bool control(ScriptImage *img, char **args);
 	virtual void down(void);
 	virtual const char *use(Line *line, ScriptImage *img);
@@ -628,6 +636,7 @@ public:
 
 	static const char *check(Line *line, ScriptImage *img);
 	static void shutdown(void);
+	static bool rebuild(ScriptCompiler *img);
 };
 
 class __EXPORT ScriptRuntime : public ScriptCommand
@@ -1067,6 +1076,16 @@ protected:
 	unsigned paren;
 	unsigned inccount;
 	const char *incfiles[256];
+
+	typedef struct _merge
+	{
+		struct _merge *next;
+		Name *target;
+		const char *source;
+		const char *prefix;
+	}	merge_t;
+
+	merge_t *mlist;
 	
 	friend class __EXPORT ScriptInterp;
 
